@@ -1,6 +1,7 @@
 ;;; nnshimbun.el --- interfacing with web newspapers
 
-;; Copyright (C) 2000,2001,2002,2003 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2000, 2001, 2002, 2003, 2004
+;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;          Akihiro Arisawa    <ari@atesoft.advantest.co.jp>,
@@ -64,6 +65,7 @@
 ;;; Code:
 
 (eval-when-compile (require 'cl))
+
 (require 'nnoo)
 (require 'nnheader)
 (require 'nnmail)
@@ -72,6 +74,8 @@
 (eval-and-compile
   (autoload 'gnus-declare-backend "gnus-start")
   (autoload 'gnus-group-make-group "gnus-group")
+  (autoload 'gnus-group-short-name "gnus")
+  (autoload 'gnus-summary-refer-article "gnus-sum")
   (autoload 'message-make-date "message")
   (autoload 'parse-time-string "parse-time"))
 
@@ -86,14 +90,17 @@
   "*If non-nil, nnshimbun will keep read articles for later re-retrieval.
 If it is a number N, then nnshimbun will only keep the last N articles
 read.  If it is neither nil nor a number, nnshimbun will keep all read
-articles.  This is not a good idea.
+articles.  That is not a good idea.
 
 Note that smaller values may spoil the `prefetch-articles' feature,
 since nnshimbun uses the backlog to keep the prefetched articles."
   :group 'nnshimbun
-  :type '(choice (const :tag "off" nil)
-		 integer
-		 (sexp :format "all"
+  :type '(choice (const :tag "Off" nil)
+		 (integer :format "%t: %v\n"
+			  :size 0)
+		 (sexp :format "All\n"
+		       :match (lambda (widget value)
+				(and value (not (numberp value))))
 		       :value t)))
 
 (defcustom nnshimbun-keep-unparsable-dated-articles t "\
@@ -133,7 +140,7 @@ since nnshimbun uses the backlog to keep the prefetched articles."
 				  :value never
 				  (const never)
 				  (const immediate)
-				  (number :tag "days"))))
+				  (integer :tag "days"))))
 	 (repeat :inline t :tag "Others"
 		 (list :inline t :format "%v"
 		       (symbol :tag "Keyword")
@@ -364,7 +371,7 @@ to SYMBOL.  If FULL-NAME-P is non-nil, it treats GROUP as a full name."
 		(dir (nnshimbun-current-directory group)))
 	    (or (file-directory-p dir)
 		(ignore-errors
-		  (make-directory dir)
+		  (make-directory dir t)
 		  (file-directory-p dir))
 		(nnheader-report 'nnshimbun
 				 (if (file-exists-p dir)
@@ -662,7 +669,7 @@ also be nil."
 		  (search-forward id nil t)) ; We find the ID.
 	;; And the id is in the fourth field.
 	(if (not (and (search-backward "\t" nil t 4)
-		      (not (search-backward "\t" (gnus-point-at-bol) t))))
+		      (not (search-backward "\t" (shimbun-point-at-bol) t))))
 	    (forward-line 1)
 	  (forward-line 0)
 	  (setq found t)))
@@ -671,7 +678,7 @@ also be nil."
 	(setq id (concat "X-Nnshimbun-Id: " id))
 	(while (and (not found)
 		    (search-forward id nil t))
-	  (if (not (search-backward "\t" (gnus-point-at-bol) t 8))
+	  (if (not (search-backward "\t" (shimbun-point-at-bol) t 8))
 	      (forward-line 1)
 	    (forward-line 0)
 	    (setq found t))))
@@ -681,7 +688,7 @@ also be nil."
 ;; This function is defined as the alternative of `nnheader-parse-nov',
 ;; in order to keep compatibility between T-gnus and Oort Gnus.
 (defun nnshimbun-parse-nov ()
-  (let ((eol (gnus-point-at-eol)))
+  (let ((eol (shimbun-point-at-eol)))
     (let ((number  (nnheader-nov-read-integer))
 	  (subject (nnheader-nov-field))
 	  (from    (nnheader-nov-field))
@@ -765,7 +772,7 @@ article to be expired.  The optional fourth argument FORCE is ignored."
 	(while expirable
 	  (setq article (pop expirable))
 	  (when (and (nnheader-find-nov-line article)
-		     (setq end (gnus-point-at-eol))
+		     (setq end (shimbun-point-at-eol))
 		     (not (= (point-max) (1+ end))))
 	    (setq time (and (search-forward "\t" end t)
 			    (search-forward "\t" end t)
@@ -799,6 +806,47 @@ article to be expired.  The optional fourth argument FORCE is ignored."
 		       id))
 
 
+;; Functions to follow anchors that point previous articles:
+
+(defun nnshimbun-search-xref (group url)
+  "Search an article generated from a given URL in the GROUP,
+and return its header."
+  (let ((buffer (nnshimbun-nov-buffer-name group)))
+    (when (gnus-buffer-live-p buffer)
+      (with-current-buffer buffer
+	(goto-char (point-min))
+	(let (found)
+	  (unless found
+	    (goto-char (point-min))
+	    (setq url (concat "Xref: " url))
+	    (while (and (not found)
+			(search-forward url nil t)
+			(looking-at "[?\t]"))
+	      (if (not (search-backward "\t" (shimbun-point-at-bol) t 7))
+		  (forward-line 1)
+		(forward-line 0)
+		(setq found t))))
+	  (when found
+	    (nnshimbun-parse-nov)))))))
+
+;;;###autoload
+(defun gnus-summary-refer-shimbun-article (url)
+  "Show a shimbun article pointed by the given URL."
+  (interactive "sURL: ")
+  (let ((method (gnus-find-method-for-group gnus-newsgroup-name))
+	(header))
+    (and (eq 'nnshimbun (car method))
+	 (nnshimbun-possibly-change-group nil (nth 1 method))
+	 (setq header (nnshimbun-search-xref
+		       (gnus-group-short-name gnus-newsgroup-name) url))
+	 (with-current-buffer gnus-summary-buffer
+	   (gnus-summary-refer-article (shimbun-header-id header))))))
+
+(defun nnshimbun-setup-article-mode ()
+  (set (make-local-variable 'w3m-goto-article-function)
+       'gnus-summary-refer-shimbun-article))
+
+
 ;; Command to create an nnshimbun group:
 
 (defvar nnshimbun-server-history nil)
