@@ -48,7 +48,9 @@
     ("geinou" "$B7]G=(B")
     ("kurashi" "$BJk$i$7(B")
     ("it" "IT")
-    ("kagaku" "$B2J3X(B")
+    ("kagaku.science" "$B2J3X(B")
+    ("kagaku.env" "$B4D6-(B")
+    ("kagaku.medical" "$B0e3X(B")
     ("seiji" "$B@/<#(B")
     ("keizai" "$B7P:Q(B")
     ("kokusai" "$B9q:](B"))
@@ -84,7 +86,8 @@ Face: iVBORw0KGgoAAAANSUhEUgAAABwAAAAcBAMAAACAI8KnAAAABGdBTUEAALGPC/xhBQAAABh
 
 (luna-define-method shimbun-index-url ((shimbun shimbun-mainichi))
   (concat (shimbun-url-internal shimbun)
-	  (shimbun-current-group-internal shimbun)
+	  (shimbun-subst-char-in-string
+	   ?. ?/ (shimbun-current-group-internal shimbun))
 	  "/"))
 
 (defun shimbun-mainichi-make-date-string (&rest args)
